(function() {
	new Vue({
		el: '#app',
		data: {
			sgf: "",
			msg: "",

			nowHands: 0,
			bcapCount: 0,
			wcapCount: 0,
			nodeCount: 0,
			curColor: 1, // 当前落子颜色
			player: null,
			tryMove: null,
			maker: null,
			moveCb: null,
			tempStone: {
				x: -2,
				y: -2
			},
			isLast: true,
			stoneList: [],
			markerConfig: {
				markerStyle: "LB", // LB
				markerNum: 1, // 5
				lastMoveColor: "red",
				start: 0
			},
			swipMove: false, // 下滑落子
			moveArea: null,
			boardError: 0, // 棋盘错误标记 0:默认值; 1:有错误; 2:有错误但不显示
		},
		mounted: function() {
			clean.uniReady(function() {
				//初始化棋盘
				this.initBoard()
				//绑定事件
				this.bindEvent();
			}.bind(this));
		},
		methods: {
			getQueryString(name) {
				var reg = new RegExp("(^|&)" + name + "=([^&]*)(&|$)", "i");
				var r = window.location.search.substr(1).match(reg);
				if (r != null) return unescape(r[2]);
				return 15;
			},

			initBoard() {
				var _this = this;
				this.player = new WGo.BasicPlayer(document.getElementById("board"), {
					sgf: "(;SZ[" + _this.getQueryString("size") + "])",
					layout: {},
					board: {
						background: "js/wgo/boardbg.png",
					},
					markLastMove: false,
					update: function(e) {
						_this.nowHands = e.path.m;
						_this.nodeCount = e.target.kifu.nodeCount;
						_this.curColor = e.target.kifuReader.game.turn;
						if (e.target.kifuReader.node.children.length == 0) {
							_this.isLast = true;
						} else {
							_this.isLast = false;
						}

						//发送消息
						uni.postMessage({
							data: {
								"type": "boardUpdate",
								"data": JSON.stringify({
									nowHands: _this.nowHands,
									bcapCount: _this.bcapCount,
									wcapCount: _this.wcapCount,
									nodeCount: _this.nodeCount,
									isLast: _this.isLast,
									curColor: _this.curColor,
									scoreSgf: e.target.kifuReader.kifu.scoreSgf(
										e.target.kifuReader.node),
									cleanSgf: e.target.kifuReader.kifu
										.cleanSgf(),
									content: e.node.comment || '',
									errorFlag: _this.boardError,
									kifuInfo: e.target.kifuReader.kifu.info
								})
							}
						});
						if (_this.isLast && _this.stoneList.length > 0 && !_this.tryMove) {
							_this.stoneList.forEach(obj => {
								if (obj.isPass) {
									setTimeout(() => {
										_this.player.pass();
									}, 50);
								} else {
									setTimeout(() => {
										_this.player.play(obj.move.x, obj.move
											.y);
									}, 50);
								}
							});
							_this.stoneList = [];
						}
					},
					error: function(e) {
						_this.boardError = e.flag
						uni.postMessage({
							data: {
								"type": "boardError",
								"data": JSON.stringify({
									errorFlag: e.flag
								})
							}
						});
					}
				});
				this.player.last();
				this.player.setCoordinates_single(!this.player.coordinate)
				this.maker = new WGo.Player.Marker(this.player, this.player.board);
				// this.maker.switchMaker(this.markerConfig)
				this.editable = new WGo.Player.Editable(this.player, this.player.board);
				this.editable.setMove(false)
				this.editable.setBoardControlCB(this.BoardControlCB.bind(this));
				this.editable.setCallback(this.playCb.bind(this));
				this.moveArea = new WGo.Player.PatternSearch(this.player, this.player.board, null);
			},

			text() {
				this.moveCb('ok')
			},

			playCb(num, stone) {
				if (num == '') {
					stone.x = -1
					stone.y = -1
				}
				if ((this.tempStone.x != stone.x || this.tempStone.y != stone.y)) {
					if (num != '') {
						let isWin = this.checkIsFive(stone)
						stone.isWon = isWin
						uni.postMessage({
							data: {
								"type": "moveDown",
								"data": JSON.stringify({
									show: true,
									moves: stone
								})
							}
						});
					}
				}
				if (this.editable.IsPro) {
					this.moveArea.drawArea(this.player)
				}
			},

			BoardControlCB(flag, moves, cb) {
				this.moveCb = cb
				if (flag) {
					let isWon = this.checkIsFive(moves)
					moves.isWon = isWon
				} else {
					moves = ''
				}
				uni.postMessage({
					data: {
						"type": "moveCb",
						"data": JSON.stringify({
							show: flag,
							moves: moves
						})
					}
				});
			},
			tryMoveF(result) {
				let data = {}
				if (result.startKomi) {
					this.maker.style = 'LA'
					data = {
						isDrop: 1,
						isPro: true,
						komiN: result.komi ? result.komi : 0,
						isPreplay: result.isPreplay
					}
				} else {
					data = {
						isDrop: 0,
						isPro: false
					}
				}
				Object.assign(this.maker, data)
				if (this.tryMove) {
					this.endTryMove()
					//处理落子
					if (this.isLast && this.stoneList.length > 0) {
						this.stoneList.forEach(obj => {
							if (obj.isPass) {
								setTimeout(() => {
									this.player.pass();
								}, 50);
							} else {
								setTimeout(() => {
									this.player.play(obj.move.x, obj.move.y);
								}, 50);
							}
						});
						this.stoneList = [];
					}
					uni.postMessage({
						data: {
							"type": "tryMove",
							"msg": '关闭试下'
						}
					});
				} else {
					this.tryMove = new WGo.TryMove(this.player, this.player.board, this.maker)
					this.tryMove.openCallBack = true
					this.tryMove.setCallback((res) => {
						uni.postMessage({
							data: {
								"type": res.type,
								"data": JSON.stringify(res)
							}
						});
					})

					if (result.stonesList) {
						result.stonesList.forEach((item) => {
							this.tryMove.play(item.x, item.y)
						})
					}
					if (!result.startKomi) {
						uni.postMessage({
							data: {
								"scoreSgf": this.player.kifuReader.kifu.scoreSgf(this.player
									.kifuReader.node),
								"type": "tryMove",
								"msg": '开启试下',
								"extra": result
							}
						});
					}
					if (result) {
						this.playTryMove(result)
					}
				}
			},

			// 结束试下
			endTryMove() {
				if (this.tryMove) {
					this.tryMove.openCallBack = false
					this.maker.style = 'LB'
					this.tryMove.endTry()
					this.tryMove = null
					this.maker.switchMaker(this.markerConfig)
					this.player.previous()
					this.player.next()
					this.moveArea.close()
				}
			},

			// 支招
			recommend(moves) {
				if (this.tryMove) {
					this.tryMove.play(moves.x, moves.y, moves.c)
				} else {
					this.tryMove = new WGo.TryMove(this.player, this.player.board, this.maker)
					uni.postMessage({
						data: {
							"scoreSgf": this.player.kifuReader.kifu.scoreSgf(this.player.kifuReader
								.node),
							"type": "tryMove",
							"msg": '开启试下',
							"extra": moves
						}
					});
					this.tryMove.play(moves.x, moves.y, moves.c)
				}
			},

			// 打点落子
			playDaDian(moves) {
				if (this.tryMove) {
					this.tryMove.play(moves.x, moves.y)
				} else {
					this.maker.isDrop = 1
					this.maker.style = 'LA'
					this.tryMove = new WGo.TryMove(this.player, this.player.board, this.maker)
					uni.postMessage({
						data: {
							"scoreSgf": this.player.kifuReader.kifu.scoreSgf(this.player.kifuReader
								.node),
							"type": "tryMove",
							"msg": '开启试下'
						}
					});
					this.tryMove.play(moves.x, moves.y)
				}
			},

			// 支招落子坐标转换
			playTryMove(result) {
				let curColor = ''
				result.moves && result.moves.forEach(val => {
					let x, y
					x = val.charCodeAt(0) - 'a'.charCodeAt(0)
					if (x < 0) x += 'a'.charCodeAt(0) - 'A'.charCodeAt(0)
					if (x > 7) x--
					y = (val.charCodeAt(1) - '0'.charCodeAt(0))
					if (val.length > 2) y = y * 10 + (val.charCodeAt(2) - '0'.charCodeAt(0))
					y = this.player.kifuReader.kifu.size - y
					let stone = {
						X: x,
						Y: y
					}
					if (!curColor) {
						curColor = result.c
					} else {
						curColor = curColor == 1 ? -1 : 1
					}
					this.tryMove.play(stone.X, stone.Y, curColor)
				})
			},

			// 校验输赢
			checkIsFive(move) {
				const {
					x,
					y,
					c
				} = move
				var boardSize = this.player.kifuReader.kifu.size
				var postion = this.player.kifuReader.game.getPosition()
				var info = this.player.kifuReader.kifu.info;

				var leftX = Math.max(x - 4, 0);
				var rightX = Math.min(x + 4, boardSize - 1);
				var topY = Math.max(y - 4, 0);
				var bottomY = Math.min(y + 4, boardSize - 1);

				if (info['RU'] === 'swap2') {
					leftX = Math.max(x - 14, 0);
					rightX = Math.min(x + 14, boardSize - 1);
					topY = Math.max(y - 14, 0);
					bottomY = Math.min(y + 14, boardSize - 1);
				}

				var horizontal = 1
				//向左
				for (var i = x - 1; i >= leftX; i--) {
					var target = y + i * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					horizontal++
				}
				//向右
				for (var i = x + 1; i <= rightX; i++) {
					var target = y + i * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					horizontal++
				}
				if (this.checkGreaterThanFive(horizontal)) return true;

				var vertical = 1
				//向上
				for (var j = y - 1; j >= topY; j--) {
					var target = j + x * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					vertical++;
				}
				//向下
				for (var j = y + 1; j <= bottomY; j++) {
					var target = j + x * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					vertical++;
				}
				if (this.checkGreaterThanFive(vertical)) return true;

				var leftOblique = 1;
				// 右斜上
				for (var i = x + 1, j = y - 1; i <= rightX && j >= topY; i++, j--) {
					var target = j + i * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					leftOblique++;
				}
				// 左斜下
				for (var i = x - 1, j = y + 1; i >= leftX && j <= bottomY; i--, j++) {
					var target = j + i * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					leftOblique++;
				}
				if (this.checkGreaterThanFive(leftOblique)) return true;

				var rightOblique = 1;
				// 左斜向上
				for (var i = x - 1, j = y - 1; i >= leftX && j >= topY; i--, j--) {
					var target = j + i * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					rightOblique++;
				}
				// 右斜向下
				for (var i = x + 1, j = y + 1; i <= rightX && j <= bottomY; i++, j++) {
					var target = j + i * boardSize;
					var color = postion.schema[target]
					if (color != c) {
						break;
					}
					rightOblique++;
				}
				if (this.checkGreaterThanFive(rightOblique)) return true;
				return false
			},

			// 校验是否大于5子
			checkGreaterThanFive(num) {
				var info = this.player.kifuReader.kifu.info
				if (info['RU'] === 'swap2') {
					if (num === 5) {
						return true
					}
				} else {
					if (num >= 5) {
						return true
					}
				}
				return false;
			},

			// 页面通信
			sendMessage(data) {
				uni.postMessage({
					data: {
						"type": "custom",
						"data": JSON.stringify(data)
					}
				});
			},

			bindEvent() {
				var _this = this
				var startPoint = null;
				document.addEventListener("dblclick", function(e) {
					uni.postMessage({
						data: {
							"type": "dbClick",
						}
					});
				})
				document.addEventListener("touchstart", function(e) {
					if (_this.swipMove) {
						var e = e || window.event;
						startPoint = e.touches[0];
					}
				})
				document.addEventListener("touchend", function(e) {
					if (!_this.swipMove) return;
					var e = e || window.event;
					//e.changedTouches能找到离开手机的手指，返回的是一个数组
					var endPoint = e.changedTouches[0];
					//计算终点与起点的差值
					var x = endPoint.clientX - startPoint.clientX;
					var y = endPoint.clientY - startPoint.clientY;
					//设置滑动距离的参考值
					var d = 10;
					if (Math.abs(y) > d) {
						if (y > 0) {
							uni.postMessage({
								data: {
									"type": "downOk",
								}
							});
						} else {
							_this.moveCb('cancel')
						}
					}
				});

				//加载棋谱
				document.addEventListener('loadSgf', function(event) {
					if (_this.tryMove) {
						_this.endTryMove()
					}
					_this.sgf = event.detail.sgf;
					_this.tempStone = {
						x: -2,
						y: -2
					};
					if (_this.boardError > 0 && event.detail.closeError === 1) {
						_this.player.closeErrorMsg()
						_this.boardError = 0
					}
					_this.player.loadSgf(_this.sgf)
					_this.player.last();
				});
				//回到第一手
				document.addEventListener('first', function(event) {
					_this.player.first();
				});
				// 后退5手
				document.addEventListener('previous5', function(event) {
					let hands = _this.player.kifuReader.path.m - 5
					_this.player.goTo(hands)
				});
				// 后退一手
				document.addEventListener('previous', function(event) {
					_this.player.previous()
				});
				// 前进一手
				document.addEventListener('next', function(event) {
					_this.player.next()
				});
				// 前进5手
				document.addEventListener('next5', function(event) {
					if (_this.player.kifuReader.path.m == _this.player.kifuReader.kifu.nodeCount)
						return;
					_this.player.goTo(_this.player.kifuReader.path.m + 5)
				});

				// 最后一手
				document.addEventListener('last', function(event) {
					_this.player.last()
				});

				// 开启试下
				document.addEventListener('tryMove', function(event) {
					_this.tryMoveF(event.detail)
				});

				// 关闭试下
				document.addEventListener('endTryMove', function(event) {
					_this.endTryMove()
				});

				// 开启棋子虚影
				document.addEventListener('setCurrent', function(event) {
					_this.editable.setCurrent(event.detail.flag)
				});

				// 支招
				document.addEventListener('recommend', function(event) {
					_this.recommend(event.detail)
				});

				// 落子微调回调
				document.addEventListener('moveStone', function(event) {
					_this.moveCb(event.detail.type)
				});

				// 切换棋盘是否可落子状态
				document.addEventListener('eventBoard', function(event) {
					_this.editable.setMove(event.detail.flag);
				});

				// 切换棋盘当前落子颜色
				document.addEventListener('setColor', function(event) {
					_this.player.kifuReader.game.turn = event.detail.color
				});

				// 切换棋盘落子状态及标记
				document.addEventListener('setBoard', function(event) {

					// 设置棋盘可落子区域
					if (event.detail.isPro) {
						let IsProCount = 0
						if (event.detail.IsProCount) IsProCount = event.detail.IsProCount
						_this.editable.setIsPro(true, IsProCount)
						_this.moveArea.drawArea(_this.player)
					} else {
						_this.editable.setIsPro(false)
						_this.moveArea.close()
					}

					if (event.detail.config) {
						_this.markerConfig = event.detail.config.boardMarker
						_this.maker.switchMaker(event.detail.config.boardMarker)
						_this.swipMove = event.detail.config.moveFlag === 3 ? true : false;
					}
				});

				// 关闭绘制
				document.addEventListener('stoneMarker', function(event) {
					if (event.detail.flag) {
						_this.moveArea.stoneMarker(event.detail.stone)
					} else {
						_this.moveArea.close();
					}
				});

				// 添加落子队列
				document.addEventListener('addStone', function(event) {
					_this.stoneList.push(event.detail.stone);
				});

				// 移除落子
				document.addEventListener('removeStone', function(event) {
					_this.player.remove()
					_this.player.kifuReader.kifu.nodeCount--
					_this.player.last()
				});

				// 清空落子队列
				document.addEventListener('cleanStone', function(event) {
					_this.stoneList = [];
				});

				// 落子
				document.addEventListener('playStone', function(event) {
					if (event.detail.stone.x == -1 && event.detail.stone.y == -1) {
						_this.editable.pass();
					} else {
						let isWon = _this.checkIsFive(event.detail.stone)
						_this.tempStone = event.detail.stone;
						_this.editable.play(event.detail.stone.x, event.detail.stone.y);
						_this.player.last()
						if (isWon) {
							_this.sendMessage({
								isWon
							})
						}
					}
				});

				// 连续pass终局
				document.addEventListener('doublePass', function(event) {
					let flag = false
					if (_this.player.kifuReader.node && _this.player.kifuReader.node.parent && _this
						.player.kifuReader.node.parent
						.move && _this.player.kifuReader.node.parent.move.pass) {
						flag = true
					}
					uni.postMessage({
						data: {
							"type": "doublePassStatus",
							"data": JSON.stringify({
								flag: flag
							})
						}
					});
				});

				// 批量删除落子
				document.addEventListener('batchRemoveStone', function(event) {
					let nodeCount = _this.player.kifuReader.kifu.nodeCount
					for (let i = 0; i < nodeCount; i++) {
						_this.player.remove()
						_this.player.kifuReader.kifu.nodeCount--
						_this.player.last()
					}
					_this.tryMove && _this.tryMove.clear()
				});

				// 专业规则打点落子
				document.addEventListener('playDaDian', function(event) {
					_this.playDaDian(event.detail)
				});

				// 
				document.addEventListener('addObject', function(event) {
					_this.player.board.addObject(event.detail);
				});

				document.addEventListener('removeObject', function(event) {
					_this.player.board.removeObject(event.detail);
				});
			}
		}
	})
})()
