(function(WGo) {
	var to_num = function(str, i) {
		return str.charCodeAt(i) - 97;
	}
	var ajaxUrl = 'https://tengen.yikeweiqi.com/score/ai';
	var ScoreModeGnugo = function(position, board, data, success, error, isShowWating) {
		this.originalPosition = position;
		this.position = position.clone();
		this.initPosition = position.clone();
		this.board = board;
		this.data = data;
		this.success = success;
		this.error = error;
		this.isShowWating = isShowWating || 'show';
	}

	var state = ScoreModeGnugo.state = {
		UNKNOWN: 0,
		BLACK_STONE: 1, // must be equal to WGo.B
		WHITE_STONE: -1, // must be equal to WGo.W
		BLACK_CANDIDATE: 2,
		WHITE_CANDIDATE: -2,
		BLACK_NEUTRAL: 3,
		WHITE_NEUTRAL: -3,
		NEUTRAL: 4,
	}

	ScoreModeGnugo.prototype.start = function(waiting) {
		this.waiting = waiting || null;
		this.showWaiting();
		this.saved_state = this.board.getState();
		this.remotescore();
	}

	ScoreModeGnugo.prototype.end = function() {
		this.board.restoreState({
			objects: WGo.clone(this.saved_state.objects)
		});
		this.closeWaiting();
	}
	ScoreModeGnugo.prototype.showWaiting = function() {
		if(this.isShowWating == 'show') {
			if(window.plus) {
				this.waiting = plus.nativeUI.showWaiting('', {
					padlock: true
				})
			} else {
				this.waiting.show();
			}
		}
	}
	ScoreModeGnugo.prototype.closeWaiting = function() {
		this.waiting && this.waiting.close();
	}
	ScoreModeGnugo.prototype.displayScore = function() {
		var score = {
			bc: [], //黑的子数
			wc: [], //白的子数
			black: [], //黑的空
			white: [], //白的空
			neutral: [], //空子
			black_captured: [], //黑色虚子
			white_captured: [], //白色虚子
		}
		for(var i = 0; i < this.position.size; i++) {
			for(var j = 0; j < this.position.size; j++) {
				s = this.position.get(i, j);
				t = this.originalPosition.get(i, j);

				if(s == state.BLACK_CANDIDATE) score.black.push({
					x: i,
					y: j,
					type: "mini",
					c: WGo.B
				});
				else if(s == state.WHITE_CANDIDATE) score.white.push({
					x: i,
					y: j,
					type: "mini",
					c: WGo.W
				});
				else if(s == state.NEUTRAL) score.neutral.push({
					x: i,
					y: j
				});
				else if(s == state.BLACK_STONE) score.bc.push({
					x: i,
					y: j,
					c: WGo.B
				});
				else if(s == state.WHITE_STONE) score.wc.push({
					x: i,
					y: j,
					c: WGo.W
				});
				else score.neutral.push({
					x: i,
					y: j
				});

				if(t == WGo.W && s != state.WHITE_STONE) score.white_captured.push({
					x: i,
					y: j,
					type: "outline",
					c: WGo.W
				});
				else if(t == WGo.B && s != state.BLACK_STONE) score.black_captured.push({
					x: i,
					y: j,
					type: "outline",
					c: WGo.B
				});
			}
		}
		for(var i = 0; i < score.black_captured.length; i++) {
			this.board.removeObjectsAt(score.black_captured[i].x, score.black_captured[i].y);
		}

		for(var i = 0; i < score.white_captured.length; i++) {
			this.board.removeObjectsAt(score.white_captured[i].x, score.white_captured[i].y);
		}
		this.board.addObject(score.white_captured);
		this.board.addObject(score.black_captured);
		this.board.addObject(score.black);
		this.board.addObject(score.white);
		var bdame = score.neutral.length;
		bdame = bdame / 2;
		var btake = (score.white_captured.length + this.originalPosition.capCount.black);
		var wtake = (score.black_captured.length + this.originalPosition.capCount.white);
		this.handicap = parseInt(this.handicap) == 1 ? 0 : parseInt(this.handicap);
		var cnresult = score.bc.length + score.black.length + bdame - parseFloat(this.komi) / 2 - this.handicap / 2 - ((this.board.size * this.board.size) / 2);
		var jpresult = (score.black.length + btake) - (score.white.length + wtake + parseFloat(this.komi));
		var jpwin = jpresult > 0 ? 1 : -1;
		var cnwin = cnresult > 0 ? 1 : -1;
		var result = {
			cn: {
				blackstone: score.bc.length + score.black.length, //黑子数
				whitestone: score.wc.length + score.white.length, //白子数
				result: Math.abs(cnresult), //数子结果
				empty: bdame, //空的子数
				win: cnwin
			},
			jp: {
				blackstone: score.black.length, //黑目
				whitestone: score.white.length, //白目
				result: Math.abs(jpresult), //数目结果
				win: jpwin
			},
			komi: parseFloat(this.komi), //贴目
			btake: btake, //黑的提子
			wtake: wtake, //白的提子
		};
		this.waiting && this.waiting.close();
		this.success(result);
	}
	ScoreModeGnugo.prototype.remotescore = function() {
		var that = this;
		var options = {
			url: ajaxUrl,
			isAllurl: true,
			data: that.data
		}
		app.postAjax(options, function(result) {
			that.handicap = result.ha;
			that.komi = result.km;
			that.calcResult(result);
		}, function(e) {
			that.failCB();
		})
	}
	ScoreModeGnugo.prototype.failCB = function() {
		this.error && this.error();
		plus.nativeUI.closeWaiting();
	}
	ScoreModeGnugo.prototype.calcResult = function(data) {
		var p = this.position;
		var score = {
			black: [],
			white: [],
			neutral: [],
			black_captured: [],
			white_captured: [],
		}
		var cb = data.b;
		var cw = data.w;
		var itema = cb.split(":");
		for(var it in itema) {
			var i = to_num(itema[it], 0);
			var j = to_num(itema[it], 1);
			s = this.originalPosition.get(i, j);
			if(s != state.BLACK_STONE) {
				p.set(i, j, state.BLACK_CANDIDATE);
			}
		}
		var itemb = cw.split(":");
		for(var it in itemb) {
			var i = to_num(itemb[it], 0);
			var j = to_num(itemb[it], 1);
			s = this.originalPosition.get(i, j);
			if(s != state.WHITE_STONE) {
				p.set(i, j, state.WHITE_CANDIDATE);
			}
		}
		this.displayScore();
	}

	ScoreModeGnugo.prototype.calculate = function() {
		var p, s, t, b, w, change;

		// 1. create testing position, empty fields has flag ScoreMode.UNKNOWN
		p = this.position;

		// 2. repeat until there is some change of state:
		change = true;
		while(change) {
			change = false;

			// go through the whole position
			for(var i = 0; i < p.size; i++) {
				//var str = "";
				for(var j = 0; j < p.size; j++) {
					s = p.get(j, i);

					if(s == state.UNKNOWN || s == state.BLACK_CANDIDATE || s == state.WHITE_CANDIDATE) {
						// get new state
						t = [p.get(j - 1, i), p.get(j, i - 1), p.get(j + 1, i), p.get(j, i + 1)];
						b = false;
						w = false;

						for(var k = 0; k < 4; k++) {
							if(t[k] == state.BLACK_STONE || t[k] == state.BLACK_CANDIDATE) b = true;
							else if(t[k] == state.WHITE_STONE || t[k] == state.WHITE_CANDIDATE) w = true;
							else if(t[k] == state.NEUTRAL) {
								b = true;
								w = true;
							}
						}

						t = false;

						if(b && w) t = state.NEUTRAL;
						else if(b) t = state.BLACK_CANDIDATE;
						else if(w) t = state.WHITE_CANDIDATE;

						if(t && s != t) {
							change = true;
							p.set(j, i, t);
						}
					}
					//str += (p.get(j,i)+5)+" ";
				}
				//console.log(str);
			}
			//console.log("------------------------------------------------------------");
		}
	}
	WGo.ScoreModeGnugo = ScoreModeGnugo;
})(WGo);